#include "Motor.h"

Motor::Motor(uint8_t r_dir_pin, uint8_t r_spd_pin, uint8_t l_dir_pin, uint8_t l_spd_pin)
{
  rightMotor.dir_pin    = r_dir_pin;
  rightMotor.speed_pin  = r_spd_pin;
  rightMotor.speed      = 0;
  rightMotor.direction  = FORWARD;
  rightMotor.adjustment = 0;
  
  leftMotor.dir_pin    = l_dir_pin;
  leftMotor.speed_pin  = l_spd_pin;
  leftMotor.speed      = 0;
  leftMotor.direction  = FORWARD;
  leftMotor.adjustment = 0;
  
  pinMode(r_dir_pin, OUTPUT);
  pinMode(r_spd_pin, OUTPUT);
  pinMode(l_dir_pin, OUTPUT);
  pinMode(l_spd_pin, OUTPUT);
  
  stop();
}

void Motor::drive(uint8_t speed, uint8_t direction)
{
  setMotor(&rightMotor, speed, direction);
  setMotor(&leftMotor, speed, direction);
}

void Motor::stop()
{
  setMotor(&rightMotor, 0, Motor::FORWARD);
  setMotor(&leftMotor, 0, Motor::FORWARD);  
}

void Motor::setMotor(motorInfo_t *motor, uint8_t speed, uint8_t direction)
{
  if ((direction == Motor::FORWARD) || (direction == Motor::BACKWARD))
  {
    motor->speed     = min(255, (speed == 0 ? 0 : (speed + motor->adjustment)));
    motor->direction = direction;
  }
  
  digitalWrite(motor->dir_pin, motor->direction);
  analogWrite(motor->speed_pin, motor->speed);
  
  DebugCode(
    Serial.print("motor: speed=");
    Serial.print(motor->speed);
    Serial.print(", dir=");
    Serial.println(motor->direction);
  )
}

void Motor::setRightAdjust(uint8_t adjust)
{
  rightMotor.adjustment = adjust;
}

void Motor::setLeftAdjust(uint8_t adjust)
{
  leftMotor.adjustment = adjust;
}
